"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var child_process = require('child_process');
/**
 * This class acts as proxy between the code to sl-node-js component.
 * */
var SlNodeJsProxy = /** @class */ (function () {
    function SlNodeJsProxy(userMessageHelper) {
        this.userMessageHelper = userMessageHelper;
        if (!userMessageHelper) {
            throw new Error("'userMessageHelper' cannot be null or undefined");
        }
    }
    SlNodeJsProxy.prototype.runConfigMode = function (configModeData, callback) {
        //TODO: Check if we need additional arguments.
        var command = "config";
        var params = {};
        var excludes = this.getBuildModeExcludeList();
        console.log("excludes:" + excludes);
        for (var p in configModeData) {
            if (excludes.indexOf(p) > -1 || p.indexOf("_") === 0 || !configModeData[p])
                continue;
            var pName = "--" + p.toLowerCase();
            params[pName] = configModeData[p];
        }
        // if (configModeData.tokenFile) {
        //     params["--tokenfile"] = configModeData.tokenFile;
        // }
        // if (configModeData.token) {
        //     params["--token"] = configModeData.token;
        // }
        // if (configModeData.appName) {
        //     params["--appname"] = configModeData.appName;
        // }
        // if (configModeData.branch) {
        //     params["--branch"] = configModeData.branch;
        // }
        // if (configModeData.build) {
        //     params["--build"] = configModeData.build;
        // }
        //
        // if (configModeData.proxy) {
        //     params["--proxy"] = configModeData.proxy;
        // }
        this.invokeCommand(command, params, callback);
    };
    SlNodeJsProxy.prototype.runScanMode = function (scanModeData, callback) {
        //TODO: Check if we need additional arguments.
        var command = "build";
        var params = {};
        var excludes = this.getBuildModeExcludeList();
        console.log("excludes:" + excludes);
        for (var p in scanModeData) {
            if (excludes.indexOf(p) > -1 || p.indexOf("_") === 0 || !scanModeData[p])
                continue;
            var pName = "--" + p.toLowerCase();
            params[pName] = scanModeData[p];
        }
        this.invokeCommand(command, params, callback);
    };
    SlNodeJsProxy.prototype.getBuildModeExcludeList = function () {
        var excludes = ["constructor"];
        return excludes;
    };
    SlNodeJsProxy.prototype.invokeCommand = function (mode, params, callback) {
        if (!mode) {
            throw new Error("'mode' cannot be null or undefined");
        }
        if (!params) {
            throw new Error("'params' cannot be null or undefined");
        }
        if (!callback) {
            throw new Error("'callback' cannot be null or undefined");
        }
        var commandParts = [];
        commandParts.push(mode);
        for (var p in params) {
            commandParts.push(p);
            commandParts.push(params[p]);
        }
        var command = this.getSlNodeJsPath();
        this.userMessageHelper.println("Command: " + command);
        this.userMessageHelper.println("Command Args: " + commandParts.join(" "));
        var slNodeJsProcess = child_process.spawn(command, commandParts, {
            stdio: 'inherit'
        });
        slNodeJsProcess.on("close", function (code) {
            var error = null;
            if (code > 0) {
                var fullCommand = command + " " + commandParts.join(" ");
                error = new Error("Failed while running sl-node-js.\nReturn Code: " + code + "\nCommand:'" + fullCommand + "'. ");
                return callback(error);
            }
            return callback(error);
        });
    };
    SlNodeJsProxy.prototype.getSlNodeJsPath = function () {
        var SL_NODE_JS_PATH = "SL_NODE_JS_PATH";
        if (process.env && process.env[SL_NODE_JS_PATH]) {
            return process.env[SL_NODE_JS_PATH];
        }
        try {
            var path = require.resolve("slnodejs");
            if (path) {
                return path;
            }
        }
        catch (e) {
        }
        return "./node_modules/.bin/slnodejs";
    };
    return SlNodeJsProxy;
}());
exports.SlNodeJsProxy = SlNodeJsProxy;
//# sourceMappingURL=SlNodeJsProxy.js.map