"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var sl_js_infra_1 = require("sl-js-infra");
var factory_1 = require("./factory");
var nycBinFile = require.resolve('nyc/bin/nyc.js');
var Module = require('module');
var patchChildProcess = require("./child-process-patcher").patchChildProcess;
var AgentLoader = /** @class */ (function () {
    function AgentLoader() {
    }
    AgentLoader.run = function (args, useIstanbul, spawnRuntimeArgs, shouldBootstrap) {
        if (shouldBootstrap === void 0) { shouldBootstrap = true; }
        var configLoader = new sl_js_infra_1.SlJsInfra.ConfigLoader();
        var agentCfg = configLoader.loadAgentConfiguration({});
        agentCfg.useIstanbul.value = useIstanbul;
        global["agentCfg"] = agentCfg;
        AgentLoader.init(agentCfg, args, spawnRuntimeArgs, shouldBootstrap);
    };
    AgentLoader.init = function (agentCfg, args, spawnRuntimeArgs, shouldBootstrap) {
        if (shouldBootstrap === void 0) { shouldBootstrap = true; }
        var factory = new factory_1.NodeAgentFactory(agentCfg);
        var logger = factory.getCreateApplicationLogger();
        if (agentCfg.enableChildProcessPatcher.value) {
            logger.warn("patchChildProcess is enabled.");
            patchChildProcess(__filename);
        }
        if (agentCfg.useIstanbul.value == true) {
            logger.warn("Running agent using Istanbul.");
            AgentLoader.runAgentUsingIstanbul(args, shouldBootstrap);
        }
        else {
            logger.info("Running agent using NYC");
            AgentLoader.runAgentUsingNyc(agentCfg, args, spawnRuntimeArgs, shouldBootstrap);
            logger.info("After Running agent using NYC");
        }
        Module.runMain();
    };
    AgentLoader.runAgentUsingNyc = function (agentCfg, args, spawnRuntimeArgs, shouldBootstrap) {
        process.env.SL_CONFIGURATION = JSON.stringify(agentCfg.toJsonObject());
        process.argv = [process.argv[0], nycBinFile, process.argv[0]].concat(spawnRuntimeArgs).concat(args);
        if (shouldBootstrap) {
            process.argv.splice(2, 0, "-i", require.resolve('./bootstrap'));
        }
    };
    AgentLoader.runAgentUsingIstanbul = function (args, shouldBootstrap) {
        if (shouldBootstrap) {
            require("./bootstrap");
        }
        var istanbulMain = require.resolve('istanbul/lib/cli.js');
        var x = require(istanbulMain); //Module.runMain();
        x.runToCompletion(['cover'].concat(args));
    };
    return AgentLoader;
}());
exports.AgentLoader = AgentLoader;
//# sourceMappingURL=agent-loader.js.map