"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var factory_1 = require("./factory");
var agent_1 = require("./agent");
var nyc_report_executor_1 = require("./nyc-report-executor");
var sl_js_infra_1 = require("sl-js-infra");
var logger_1 = require("./logger");
var upload_report_handler_1 = require("./upload-report-handler");
var LOGGER = new logger_1.LoggerFactory().getCreateApplicationLogger();
function init(runtimeCfg) {
    return initAsync(runtimeCfg, function () {
    });
}
exports.init = init;
function initAsync(runtimeCfg, callback, toolInfo) {
    var configLoader = new sl_js_infra_1.SlJsInfra.ConfigLoader();
    var agentCfg = configLoader.loadAgentConfiguration(runtimeCfg && runtimeCfg.toJsonObject && runtimeCfg.toJsonObject());
    var factory = new factory_1.NodeAgentFactory(agentCfg);
    var agent = factory.getCreateNodeJsAgent();
    agent.agentEventsController.addTool(toolInfo);
    agent.start(function (err) {
        callback(err, agent);
    });
    return agent;
}
exports.initAsync = initAsync;
function getVersion() {
    return agent_1.NodeJsAgent.getVersion();
}
exports.getVersion = getVersion;
function uploadNycReport(reportFile, agentCfg) {
    var executor = new nyc_report_executor_1.NycReportExecutor(reportFile, agentCfg);
    return executor.execute();
}
exports.uploadNycReport = uploadNycReport;
function startExecution(args, httpConfig, callback) {
    LOGGER.info("[EXECUTION SERVICE] about to send start execution request");
    LOGGER.debug("[START EXECUTION] execution request params are: %s", JSON.stringify(args));
    var requestCallback = function (err, body) {
        if (err) {
            LOGGER.error("[START EXECUTION] error while starting execution '%s'", err);
            callback(err);
        }
        else {
            LOGGER.info("[START EXECUTION] execution started successfully");
            callback(null);
        }
    };
    var backEndProxy = new sl_js_infra_1.SlJsInfra.BackendProxy(httpConfig, LOGGER);
    backEndProxy.startExecution(args, requestCallback);
}
exports.startExecution = startExecution;
function endExecution(args, httpConfig, callback) {
    LOGGER.info("[EXECUTION SERVICE] about to send end execution request");
    LOGGER.debug("[END EXECUTION] execution request params are: %s", JSON.stringify(args));
    var requestCallback = function (err, body) {
        if (err) {
            LOGGER.error("[END EXECUTION] error while trying to end execution '%s'", err);
            callback(err);
        }
        else {
            LOGGER.info("[END EXECUTION] execution ended successfully");
            callback(null);
        }
    };
    var backEndProxy = new sl_js_infra_1.SlJsInfra.BackendProxy(httpConfig, LOGGER);
    backEndProxy.endExecution(args, requestCallback);
}
exports.endExecution = endExecution;
function uploadReports(args, httpConfig, agentData, callback) {
    LOGGER.info("[UPLOAD REPORTS] about to upload reports");
    LOGGER.debug("[UPLOAD REPORTS] request params are: %s", JSON.stringify(args));
    var requestCallback = function (err) {
        if (err) {
            LOGGER.error("[UPLOAD REPORTS] error while trying to upload reports '%s'", err);
            callback(err);
        }
        else {
            LOGGER.info("[UPLOAD REPORTS] reports uploaded successfully");
            callback(null);
        }
    };
    var backEndProxy = new sl_js_infra_1.SlJsInfra.BackendProxy(httpConfig, LOGGER);
    var handler = new upload_report_handler_1.UploadReportHandler(args, backEndProxy, agentData);
    handler.upload(requestCallback);
}
exports.uploadReports = uploadReports;
function externalReport(args, httpConfig, agentData, callback) {
    LOGGER.info("[UPLOAD EXTERNAL REPORT] about to send end execution request");
    LOGGER.debug("[UPLOAD EXTERNAL REPORT] request params are: %s", JSON.stringify(args));
    var requestCallback = function (err, body) {
        if (err) {
            LOGGER.error("[UPLOAD EXTERNAL REPORT] error while trying to upload external report '%s'", err);
            callback(err);
        }
        else {
            LOGGER.info("[UPLOAD EXTERNAL REPORT] report uploaded successfully");
            callback(null);
        }
    };
    var backEndProxy = new sl_js_infra_1.SlJsInfra.BackendProxy(httpConfig, LOGGER);
    backEndProxy.externalReport(agentData, requestCallback);
}
exports.externalReport = externalReport;
function getConfiguration(configFile) {
    var configLoader = new sl_js_infra_1.SlJsInfra.ConfigLoader();
    return configLoader.loadAgentConfiguration({});
}
//# sourceMappingURL=api.js.map