"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var events = require("events");
var config_system_1 = require("./config-system");
var assign = require("object-assign");
var jwtDecode = require("jwt-decode");
var ConfigProcess = /** @class */ (function (_super) {
    __extends(ConfigProcess, _super);
    function ConfigProcess(cfg, watchdog, configServiceProxy, logger) {
        var _this = _super.call(this) || this;
        _this.cfg = cfg;
        _this.watchdog = watchdog;
        _this.configServiceProxy = configServiceProxy;
        _this.logger = logger;
        _this.isRunning = false;
        if (!cfg)
            throw new Error("cfg is required");
        _this.initialCfg = assign({}, cfg.toJsonObject());
        if (!watchdog)
            throw new Error("watchdog is required");
        if (!configServiceProxy)
            throw new Error("configServiceProxy is required");
        if (!logger) {
            throw new Error("logger is required");
        }
        watchdog.on("alarm", function () {
            watchdog.stop();
            _this.reloadConfigFromServer();
        });
        _this.initCfg();
        return _this;
    }
    ConfigProcess.prototype.reloadConfigFromServer = function (callback) {
        var _this = this;
        this.configServiceProxy.getConfigurationFromServer(function (err, updatedCfg) {
            if (!err && updatedCfg) {
                _this.mergeConfigFromServerAndFireEvent(updatedCfg);
            }
            _this.watchdog.start();
            if (callback) {
                callback(err);
            }
        });
    };
    ConfigProcess.prototype.mergeConfigFromServerAndFireEvent = function (updatedCfgObject) {
        var configChanged = false;
        //Load values before any server overrides
        this.cfg.loadConfiguration(new config_system_1.JsonObjectConfigurationProvider(this.initialCfg));
        for (var key in updatedCfgObject) {
            if (this.cfg[key] && this.cfg[key].isConfigKey && this.cfg[key].value != updatedCfgObject[key]) {
                configChanged = true;
                break;
            }
        }
        if (configChanged) {
            //Then override with server sent config
            this.cfg.loadConfiguration(new config_system_1.JsonObjectConfigurationProvider(updatedCfgObject));
            this.emit("configuration_changed", this.cfg);
        }
    };
    ConfigProcess.prototype.getConfiguration = function () {
        return this.cfg;
    };
    ConfigProcess.prototype.initCfg = function () {
        if (this.cfg.token.hasValue) {
            try {
                var tokenData = jwtDecode(this.cfg.token.value);
                if (!tokenData["x-sl-server"]) {
                    throw new Error("Token Is Invalid. Doesn't Contain Server");
                }
                if (!this.cfg.server.hasValue)
                    this.cfg.server.value = tokenData["x-sl-server"];
                if (!tokenData["subject"]) {
                    throw new Error("Token Is Invalid. Doesn't Contain Subject");
                }
                var customerId = tokenData["subject"];
                var subjectParts = tokenData["subject"].split("@");
                customerId = subjectParts[0];
                this.cfg.customerId.value = this.cfg.customerId.value || customerId;
                //this.cfg.interval = this.cfg.interval || 10000;
            }
            catch (err) {
                this.cfg.enabled.value = false;
                this.logger.error("Error loading configuration. Agent will be disabled.", err);
            }
        }
    };
    ConfigProcess.prototype.start = function (callback) {
        if (this.isRunning)
            return; //Already started
        if (!this.cfg.enabled.value)
            return; //Don't do anything if agent is completely disabled
        if (!this.cfg.server.value || !this.cfg.token.hasValue)
            return; //Can't start without server or token        
        this.reloadConfigFromServer(callback);
        //this.watchdog.start(); //Will be called on the first reload, which is called from agent.start()
        this.isRunning = true;
    };
    ConfigProcess.prototype.stop = function (callback) {
        this.watchdog.stop();
        this.isRunning = false;
        return callback();
    };
    return ConfigProcess;
}(events.EventEmitter));
exports.ConfigProcess = ConfigProcess;
//# sourceMappingURL=index.js.map