"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Created by nadav on 21/03/2017.
 */
var stringify = require('querystring').stringify;
var path = require('path');
var fs = require('fs');
var async = require('async');
var base_proxy_1 = require("../base-proxy");
var request = require("request");
var ExecutionServiceProxy = /** @class */ (function (_super) {
    __extends(ExecutionServiceProxy, _super);
    function ExecutionServiceProxy(cfg, request, logger) {
        return _super.call(this, cfg, request, logger) || this;
    }
    //var log = logger? logger.child({className: "ExecutionService"}) : consoleFileLogger("ExecutionService");
    ExecutionServiceProxy.prototype.startExecution = function (config, callback) {
        var data = {
            "customerId": config.customerId,
            "appName": config.appName,
            "newEnvironment": config.testStage,
            "source": config.source
        };
        if (config.labId) {
            data.environment = config.labId;
        }
        if (config.build) {
            data.buildName = config.build;
        }
        if (config.branch) {
            data.branchName = config.branch;
        }
        var routeVersion = config.isValidToken ? "/v3" : "/v2";
        var relativeUrl = routeVersion + '/testExecution';
        var bufferToSend = new Buffer(JSON.stringify(data));
        this.logger.info("Start execution. Url:'" + relativeUrl + "', Data:", data);
        this.post(bufferToSend, relativeUrl, callback);
    };
    ;
    ExecutionServiceProxy.prototype.endExecution = function (config, callback) {
        var queryParams = {
            "customerId": config.customerId.valueOf,
            "appName": config.appName
        };
        if (config.build) {
            queryParams.buildName = config.build;
        }
        if (config.branch) {
            queryParams.branchName = config.branch;
        }
        if (config.labId) {
            queryParams.environment = config.labId;
        }
        var routeVersion = config.isValidToken ? "/v3" : "/v2";
        var relativeUrl = routeVersion + '/testExecution';
        relativeUrl = relativeUrl.replace(/\?.*|$/, '?' + stringify(queryParams));
        this.logger.info("End execution. url:", relativeUrl);
        this.delete(relativeUrl, callback);
    };
    ;
    ExecutionServiceProxy.prototype.uploadExternalReport = function (config, data, callback) {
        var relativeUrl = '/v1/externalreport';
        var bufferToSend = new Buffer(JSON.stringify(data));
        this.logger.info("Uploading external report", data);
        this.post(bufferToSend, relativeUrl, callback);
    };
    ;
    ExecutionServiceProxy.prototype.uploadReport = function (config, agentData, reportFile, callback) {
        if (!reportFile) {
            return callback(new Error("'reportFile' can't be null or undefined"));
        }
        if (!reportFile.buffer) {
            return callback(new Error("'reportFile.buffer' can't be null or undefined"));
        }
        var routeVersion = config.isValidToken ? "/v3" : "/v2";
        var relativeUrl = routeVersion + '/externaldata';
        var req = request.post(relativeUrl, function (err, resp, body) {
            if (!err && resp && resp.statusCode >= 400) {
                err = new Error("HTTP ERROR: " + resp.statusCode + " " + resp.statusMessage);
            }
            if (err) {
                this.logger.error("[UPLOAD REPORTS] Error Uploading Report File: " + err + ". Report File: " + reportFile.filename);
                callback(err);
            }
            else {
                this.logger.info("[UPLOAD REPORTS] Uploaded Report File Successfully. Report File: " + reportFile.filename);
                callback(null);
            }
        });
        //When I was learning node and the request module, I was confused as to why the form could be modified
        // after the post method was called. Buried in the request docs is the explanation - "the form can be modified
        // until the request is fired on the next cycle of the event-loop"
        var form = req.form();
        form.append('file', JSON.stringify(agentData), {
            filename: 'agentData',
            contentType: 'multipart/form-data'
        });
        form.append('file', reportFile.buffer.toString(), {
            filename: 'agentData',
            contentType: 'multipart/form-data'
        });
    };
    ;
    ExecutionServiceProxy.prototype.uploadReports = function (config, callback) {
        var context = this;
        var agentData = this.getAgentData(config);
        try {
            var files = context.getAllFiles(config.reportFile, config.reportFilesFolder);
            if (files.length == 0) {
                this.logger.info("No report files found");
                return callback(null);
            }
            async.forEachLimit(files.slice(0, -1), 1, function (reportFile, callback) {
                context.uploadReport(config, agentData, reportFile, function (err) {
                    if (err) {
                        callback(err);
                    }
                    else {
                        callback(null);
                    }
                });
            }, function (err) {
                if (err) {
                    return callback(err);
                }
                var lastFile = files[files.length - 1];
                agentData.hasMoreRequests = config.hasMoreRequests;
                context.uploadReport(config, agentData, lastFile, function (err) {
                    if (err) {
                        callback(err);
                    }
                    else {
                        callback(null);
                    }
                });
            });
        }
        catch (err) {
            callback(err);
        }
    };
    ;
    ExecutionServiceProxy.prototype.getAllFiles = function (reportFiles, reportFilesFolders) {
        var _this = this;
        var files = [];
        // handle reportFilesFolders
        reportFilesFolders.forEach(function (reportFilesFolder) {
            try {
                var folderFiles = fs.readdirSync(reportFilesFolder);
                folderFiles.forEach(function (folderFileName) {
                    var stat = fs.statSync(path.join(reportFilesFolder, folderFileName));
                    if (stat.isFile()) {
                        files.push({ filename: folderFileName, buffer: fs.readFileSync(path.join(reportFilesFolder, folderFileName)) });
                    }
                    else {
                        this.logger.warn("[UPLOAD REPORTS] " + folderFileName + " is a directory. Ignoring...");
                    }
                });
            }
            catch (err) {
                _this.logger.error("[UPLOAD REPORTS] Failed Reading Folder: " + reportFilesFolder);
            }
        });
        // handle reportFiles
        reportFiles.forEach(function (reportFilePath) {
            if (fs.existsSync(reportFilePath)) {
                files.push({ filename: path.basename(reportFilePath), buffer: fs.readFileSync(reportFilePath) });
            }
            else {
                _this.logger.warn("[UPLOAD REPORTS] reportFile doesn't exist. report file: " + reportFilePath);
            }
        });
        return files;
    };
    ;
    ExecutionServiceProxy.prototype.getAgentData = function (config) {
        var agentData = {
            customerId: config.customerId,
            appName: config.appName,
            environment: {
                labId: config.labId,
                environmentName: config.environmentName
            },
            type: config.type,
            source: config.source,
            hasMoreRequests: true,
        };
        if (config.build) {
            agentData.buildName = config.build;
        }
        if (config.branch) {
            agentData.branchName = config.branch;
        }
        return agentData;
    };
    ;
    return ExecutionServiceProxy;
}(base_proxy_1.BaseProxy));
exports.ExecutionServiceProxy = ExecutionServiceProxy;
//# sourceMappingURL=execution-service-proxy.js.map