"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var path = require("path");
var sl_js_infra_1 = require("sl-js-infra");
var GLOBAL_ISTANBUL_CONTAINER_NAMES = ["__coverage__"];
function newNumberArray(length) {
    var arr = [];
    for (var i = 0; i < length; i++)
        arr.push(0);
    return arr;
}
var IstanbulToFootprintsBase = /** @class */ (function () {
    function IstanbulToFootprintsBase(logger) {
        this.previousIstanbulCounters = {};
        this.relativeModulePaths = {};
        this.logger = logger;
    }
    IstanbulToFootprintsBase.prototype.getIstanbulCounters = function () {
        if (this.globalCoverageContainer)
            return this.globalCoverageContainer;
        var re = /^\$\$cov_[0-9]+\$\$$/;
        var keys = Object.keys(global);
        if (this.logger.debug()) {
            this.logger.debug("The 'global' object contains the following keys:\n\t" + keys.join("\n\t"));
        }
        ;
        var m;
        for (var i = 0; i < keys.length; i++) {
            var k = keys[i];
            if ((m = re.exec(k)) !== null || GLOBAL_ISTANBUL_CONTAINER_NAMES.indexOf(k) >= 0) {
                this.globalCoverageContainer = global[k];
                return this.globalCoverageContainer;
            }
        }
        return null;
    };
    IstanbulToFootprintsBase.prototype.getRelativeModulePath = function (module) {
        if (!this.relativeModulePaths[module.path]) {
            this.relativeModulePaths[module.path] = path.relative(process.cwd(), module.path).replace(/\\/g, '/');
        }
        return this.relativeModulePaths[module.path];
    };
    IstanbulToFootprintsBase.prototype.formatLoc = function (loc) {
        return loc.line + ',' + loc.column;
    };
    IstanbulToFootprintsBase.prototype.getCurrentFootprints = function () {
        var diff = this.getIstanbulCountersDiff(this.getIstanbulCounters());
        return diff;
    };
    IstanbulToFootprintsBase.prototype.getIstanbulCountersDiff = function (currentCounters) {
        if (!currentCounters) {
            this.logger.error("Couldn't get istanbul counters, return empty diff.");
            return {};
        }
        var diff = {};
        var diffModule = null;
        function createModuleForDiff(istanbulModule) {
            diffModule = diff[istanbulModule] = { b: {}, f: {}, s: {} };
            return diffModule;
        }
        var prevCounters = this.previousIstanbulCounters;
        for (var istanbulModule in currentCounters) {
            diffModule = null;
            if (!prevCounters[istanbulModule]) {
                prevCounters[istanbulModule] = { b: {}, f: {}, s: {} };
            }
            var prevModule = prevCounters[istanbulModule];
            for (var funcId in currentCounters[istanbulModule].f) {
                if (prevModule.f[funcId] === undefined) {
                    prevModule.f[funcId] = 0;
                }
                var currentFunctionHits = currentCounters[istanbulModule].f[funcId];
                var prevFunctionHits = prevModule.f[funcId] || 0;
                if (currentFunctionHits > prevFunctionHits) {
                    //this is a diff
                    diffModule = diffModule || createModuleForDiff(istanbulModule);
                    diffModule.f[funcId] = (currentFunctionHits - prevFunctionHits);
                    prevModule.f[funcId] = currentFunctionHits;
                }
            }
            for (var branchId in currentCounters[istanbulModule].b) {
                var currentBranchesHits = currentCounters[istanbulModule].b[branchId];
                var prevBranchHits = prevModule.b[branchId];
                if (prevBranchHits) {
                    for (var i = 0, j = currentBranchesHits.length; i < j; i++) {
                        var currentHits = currentBranchesHits[i], previousHits = prevBranchHits[i] || 0;
                        if (currentHits > previousHits) {
                            //this is a diff
                            diffModule = diffModule || createModuleForDiff(istanbulModule);
                            diffModule.b[branchId] = diffModule.b[branchId] || newNumberArray(currentBranchesHits.length);
                            diffModule.b[branchId][i] = currentHits - previousHits;
                            prevBranchHits[i] = currentHits;
                        }
                    }
                }
                else {
                    prevModule.b[branchId] = currentBranchesHits.slice();
                    var branchSetHasHits = false;
                    for (var i = 0, j = currentBranchesHits.length; i < j && !branchSetHasHits; i++) {
                        if (currentBranchesHits[i])
                            branchSetHasHits = true;
                    }
                    if (branchSetHasHits) {
                        diffModule = diffModule || createModuleForDiff(istanbulModule);
                        diffModule.b[branchId] = currentBranchesHits.slice();
                    }
                }
            }
        }
        this.logDiff(diff);
        return diff;
    };
    IstanbulToFootprintsBase.prototype.logDiff = function (diff) {
        if (!this.logger.debug()) {
            return;
        }
        var modules = Object.keys(diff);
        var formatter = new sl_js_infra_1.SlJsInfra.MessageFormatter();
        modules.forEach(function (module) {
            var methods = diff[module].f || {};
            var branches = diff[module].b || {};
            var methodsSize = Object.keys(methods).length;
            var branchesSize = Object.keys(branches).length;
            formatter.addContent(module + ": " + methodsSize + " methods and " + branchesSize + " branches");
        });
        this.logger.debug("Found coverage data for %s files:\n %s", modules.length, formatter.content);
    };
    return IstanbulToFootprintsBase;
}());
exports.IstanbulToFootprintsBase = IstanbulToFootprintsBase;
//# sourceMappingURL=istanbul-to-footprints-base.js.map