"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var istanbul_to_footprints_base_1 = require("./istanbul-to-footprints-base");
var fs = require("fs");
var path = require("path");
var sourceMap = require("source-map");
var IstanbulToFootprintsV3 = /** @class */ (function (_super) {
    __extends(IstanbulToFootprintsV3, _super);
    function IstanbulToFootprintsV3(cfg, logger) {
        var _this = _super.call(this, logger) || this;
        _this.cfg = cfg;
        _this.totalTests = 0;
        if (!logger)
            throw new Error("logger is required");
        if (!cfg)
            throw new Error("cfg is required");
        return _this;
    }
    IstanbulToFootprintsV3.prototype.resetState = function () {
        this.totalTests = 0;
        //Holds the a mapping between a uniqueId for an element in a test, to an index in this elements 'hits' array.
        this.eidToHitsIndex = {};
        this.uniqueIdToElement = {};
        this.testNameToTestData = {};
        this.fileNameToAppFile = {};
        this.fileToSourceMapConsumer = {};
    };
    IstanbulToFootprintsV3.prototype.createFootprintsPackageForSending = function (queuedFootprints, extraData) {
        var _this = this;
        this.resetState();
        var result = this.createFootprintsFile(extraData);
        var app = result.apps[0]; //We assume single App for now.
        var istanbulMaps = this.getIstanbulCounters();
        var context = this;
        queuedFootprints.forEach(function (fp) {
            var test = _this.getOrCreateTestData(fp.testName, fp.executionId, fp.localTime, result);
            for (var moduleName in fp.footprints) {
                var istanbulModule = fp.footprints[moduleName];
                var moduleWithMaps = istanbulMaps[moduleName];
                _this.logger.debug("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
                _this.logger.debug("moduleName: ", moduleName);
                if (!moduleWithMaps) {
                    continue;
                }
                var currentTest = fp.testName;
                var relPath = _this.getRelativeModulePath(moduleWithMaps);
                if (!context.fileToSourceMapConsumer[relPath]) {
                    context.fileToSourceMapConsumer[relPath] = _this.readSourceMaps(relPath);
                    _this.logger.debug("Read source maps for file':" + relPath + "'");
                }
                var sourceMap_1 = _this.fileToSourceMapConsumer[relPath];
                var file = _this.getOrCreateFootprintsAppFile(relPath, app);
                if (istanbulModule.f) {
                    for (var funcId in istanbulModule.f) {
                        var hits = istanbulModule.f[funcId];
                        if (hits > 0) {
                            var funcInfo = moduleWithMaps.fnMap[funcId];
                            var start = funcInfo.loc.start;
                            if (funcInfo.decl) {
                                start = funcInfo.decl.start; //For NYC
                            }
                            var sourceMapData = _this.readSourceMapData(sourceMap_1, start, relPath);
                            var filePath = relPath;
                            if (sourceMapData) {
                                filePath = sourceMapData.originalFilename;
                                start = sourceMapData.originalStart;
                            }
                            var uniqueId = filePath + "@" + _this.formatLoc(start);
                            var methodElement = _this.getOrCreateMethodElement(uniqueId, file);
                            if (_this.cfg.extendedFootprints.value) {
                                methodElement.funcInfo = JSON.stringify(funcInfo);
                                methodElement.sourceMapData = JSON.stringify(sourceMapData);
                            }
                            _this.addOrUpdateElementHits(test, uniqueId, methodElement, hits);
                        }
                    }
                }
                if (istanbulModule.b) {
                    for (var branchId in istanbulModule.b) {
                        var ifElseAndSwitchCasesArr = istanbulModule.b[branchId];
                        var expressionInfo = moduleWithMaps.branchMap[branchId];
                        for (var idx = 0; idx < ifElseAndSwitchCasesArr.length; idx++) {
                            var branchInfo = expressionInfo.locations[idx];
                            var hits = ifElseAndSwitchCasesArr[idx];
                            if (hits > 0) {
                                var start = branchInfo.start;
                                var sourceMapData = _this.readSourceMapData(sourceMap_1, start, relPath);
                                var filePath = relPath;
                                if (sourceMapData) {
                                    filePath = sourceMapData.originalFilename;
                                    start = sourceMapData.originalStart;
                                }
                                var uniqueId = filePath + "|" + _this.formatLoc(start) + "|" + idx;
                                var branchElement = _this.getOrCreateBranchElement(uniqueId, file);
                                if (_this.cfg.extendedFootprints.value) {
                                    branchElement.branchInfo = JSON.stringify(branchInfo);
                                    branchElement.sourceMapData = JSON.stringify(sourceMapData);
                                }
                                _this.addOrUpdateElementHits(test, uniqueId, branchElement, hits);
                            }
                        }
                    }
                }
            }
        });
        return result;
    };
    IstanbulToFootprintsV3.prototype.addOrUpdateElementHits = function (testData, elementUniqueId, element, hits) {
        var HITS_INDEX = 1;
        var eid = elementUniqueId + "|" + testData.index;
        var index = this.eidToHitsIndex[eid];
        if (index != null) {
            var testIndexAndHits = element.hits[index];
            var elementHits = testIndexAndHits[HITS_INDEX];
            elementHits = elementHits + hits;
            testIndexAndHits[HITS_INDEX] = elementHits;
        }
        else {
            var testIndexAndHits = new Array();
            testIndexAndHits.push(testData.index);
            testIndexAndHits.push(hits);
            var length = element.hits.push(testIndexAndHits);
            this.eidToHitsIndex[eid] = length - 1;
        }
    };
    IstanbulToFootprintsV3.prototype.getOrCreateTestData = function (testName, executionId, localTime, result) {
        if (!this.testNameToTestData[testName]) {
            var test_1 = {
                executionId: executionId,
                localTime: localTime,
                testName: testName
            };
            result.tests.push(test_1);
            this.testNameToTestData[testName] = {
                index: this.totalTests++,
                test: test_1
            };
        }
        return this.testNameToTestData[testName];
    };
    IstanbulToFootprintsV3.prototype.getOrCreateMethodElement = function (uniqueId, file) {
        if (!this.uniqueIdToElement[uniqueId]) {
            var x = {
                hits: new Array(),
                uniqueId: uniqueId
            };
            this.uniqueIdToElement[uniqueId] = x;
            file.methods.push(x);
        }
        return this.uniqueIdToElement[uniqueId];
    };
    IstanbulToFootprintsV3.prototype.getOrCreateBranchElement = function (uniqueId, file) {
        if (!this.uniqueIdToElement[uniqueId]) {
            var x = {
                hits: new Array(),
                uniqueId: uniqueId
            };
            this.uniqueIdToElement[uniqueId] = x;
            file.branches.push(x);
        }
        return this.uniqueIdToElement[uniqueId];
    };
    IstanbulToFootprintsV3.prototype.getOrCreateFootprintsAppFile = function (fileName, app) {
        if (this.fileNameToAppFile[fileName]) {
            return this.fileNameToAppFile[fileName];
        }
        //File doesn't exist in request.
        var file = {
            methods: new Array(),
            branches: new Array(),
            lines: new Array(),
            path: fileName
        };
        app.files.push(file);
        this.fileNameToAppFile[fileName] = file;
        return file;
    };
    IstanbulToFootprintsV3.prototype.createFootprintsFile = function (extraData) {
        //Supporting only one app right now
        var app = {
            appName: this.cfg.appName.value,
            branchName: this.cfg.branch.value,
            buildName: this.cfg.build.value,
            moduleName: null,
            files: new Array()
        };
        extraData.apps.push(app);
        return extraData;
    };
    IstanbulToFootprintsV3.prototype.readSourceMaps = function (fullFilename) {
        if (!fullFilename)
            return;
        var sourceMapsFilename = fullFilename + '.map';
        var contents, rawSourceMapJsonData, consumer;
        try {
            contents = fs.readFileSync(sourceMapsFilename).toString();
        }
        catch (e) {
            //Sourcemap file not found
            return null;
        }
        try {
            rawSourceMapJsonData = JSON.parse(contents);
        }
        catch (e) {
            console.error('[Sealights] Ignoring invalid source map: ' + sourceMapsFilename);
            return null;
        }
        try {
            //console.log('Procssing source maps from file: '+sourceMapsFilename);
            consumer = new sourceMap.SourceMapConsumer(rawSourceMapJsonData);
            return consumer;
        }
        catch (e) {
            console.error('[Sealights] Error processing source maps for file: ' + sourceMapsFilename);
            return null;
        }
    };
    IstanbulToFootprintsV3.prototype.readSourceMapData = function (sourceMaps, start, coveragePath) {
        if (!sourceMaps)
            return null;
        try {
            //console.log('Checking original position for '+name+' at'+JSON.stringify(start));
            var originalPosition = sourceMaps.originalPositionFor(start); //{ line: start.line, column: start[1] });
            //console.log('Original position: '+JSON.stringify(originalPosition));
            if (originalPosition && originalPosition.source && originalPosition.line !== null && originalPosition.column !== null) {
                var originalFilename = originalPosition.source;
                if (originalFilename.indexOf("webpack:///webpack/") === -1 &&
                    originalFilename.indexOf("webpack:///external") === -1) { //Ignore embedded and external module
                    if (originalFilename.indexOf('webpack:///') === 0) {
                        originalFilename = originalFilename.substring(11); //'webpack:///'.length
                    }
                    originalFilename = this.fixRelPath(coveragePath, originalFilename);
                    var originalStart = { line: originalPosition.line, column: originalPosition.column };
                    var result = { originalFilename: originalFilename, originalStart: originalStart };
                    return result;
                }
            }
        }
        catch (e) {
            console.error(e);
        }
        return null;
    };
    IstanbulToFootprintsV3.prototype.fixRelPath = function (p1, p2) {
        try {
            if (!p1 || p1.length == 0)
                return p1;
            if (!p2 || p2.length == 0)
                return p1;
            //Make sure both paths are unix-style
            p1 = this.adjustPathSlashes(p1);
            p2 = this.adjustPathSlashes(p2);
            if (p2[0] == '/')
                return p2; //Absolute path
            var parts1 = p1.split('/');
            parts1.pop(); //remove the filename
            var parts2 = p2.split('/');
            var p3 = parts1.concat(parts2);
            var finalPath = this.adjustPathSlashes(path.normalize(p3.join('/')));
            return finalPath;
        }
        catch (err) {
            //log.error("Failed to fix real path for source maps. Error: " + err + ". Original File Name: " + p1 + ". Function: " + p2);
            return p2;
        }
    };
    IstanbulToFootprintsV3.prototype.adjustPathSlashes = function (filePath) {
        filePath = (filePath || "").replace(/\\/g, '/');
        return filePath;
    };
    return IstanbulToFootprintsV3;
}(istanbul_to_footprints_base_1.IstanbulToFootprintsBase));
exports.IstanbulToFootprintsV3 = IstanbulToFootprintsV3;
//# sourceMappingURL=istanbul-to-footprints-v3.js.map