"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Created by nadav on 12/03/2017.
 */
var ScanModeData_1 = require("../../entities/build-scanner/ScanModeData");
var ConfigModeData_1 = require("../../entities/build-scanner/ConfigModeData");
var SlNodeJsProxy_1 = require("../SlNodeJsProxy");
var UserMessageHelper_1 = require("../UserMessageHelper");
var AppNameResolver_1 = require("../resolvers/AppNameResolver");
var BuildResolver_1 = require("../resolvers/BuildResolver");
var BuildSessionIdResolver_1 = require("../resolvers/BuildSessionIdResolver");
var BuildSessionIdFileResolver_1 = require("../resolvers/BuildSessionIdFileResolver");
var TokenResolver_1 = require("../resolvers/TokenResolver");
var ProxyResolver_1 = require("../resolvers/ProxyResolver");
var objectAssign = require('object-assign');
// gulp-util is used to created well-formed plugin errors
var gutil = require('gulp-util');
/**
 * This class integrates Sealights into Gulp.
 * Among its main responsibilities, it runs the build-scanner and test-listener.
 * */
var SlGulp = /** @class */ (function () {
    function SlGulp(slNodeJsProxy) {
        this.slNodeJsProxy = slNodeJsProxy;
        this.userMessageHelper = new UserMessageHelper_1.UserMessageHelper("[SL-GULP]", false);
        this.appNameResolver = new AppNameResolver_1.AppNameResolver(this.userMessageHelper);
        this.buildResolver = new BuildResolver_1.BuildResolver(this.userMessageHelper);
        this.buildSessionIdResolver = new BuildSessionIdResolver_1.BuildSessionIdResolver(this.userMessageHelper);
        this.buildSessionIdFileResolver = new BuildSessionIdFileResolver_1.BuildSessionIdFileResolver(this.userMessageHelper);
        this.tokenResolver = new TokenResolver_1.TokenResolver(this.userMessageHelper);
        this.proxyResolver = new ProxyResolver_1.ProxyResolver(this.userMessageHelper);
        if (!slNodeJsProxy) {
            this.slNodeJsProxy = new SlNodeJsProxy_1.SlNodeJsProxy(this.userMessageHelper);
        }
    }
    SlGulp.prototype.runConfigMode = function (configModeData, callback) {
        if (!this.isEnabled()) {
            return callback(null);
        }
        this.userMessageHelper.println("About to run config mode.");
        if (!configModeData) {
            configModeData = new ConfigModeData_1.ConfigModeData();
        }
        if (!callback) {
            throw new Error("'callback' cannot be null or undefined");
        }
        configModeData = objectAssign(new ConfigModeData_1.ConfigModeData(), configModeData);
        configModeData.appName = this.appNameResolver.resolve(configModeData.appName);
        configModeData.build = this.buildResolver.resolve(configModeData.build);
        configModeData.buildSessionId = this.buildSessionIdResolver.resolve(configModeData.buildSessionId);
        configModeData.buildSessionIdFile = this.buildSessionIdFileResolver.resolve(configModeData.buildSessionIdFile);
        configModeData.token = this.tokenResolver.resolve(configModeData.token, configModeData.tokenFile);
        configModeData.proxy = this.proxyResolver.resolve(configModeData.proxy);
        this.slNodeJsProxy.runConfigMode(configModeData, function (error) {
            if (error) {
                var e = gutil.PluginError("gulp-slnodejs", "SlNodeJS failed while running 'config' mode. Error: " + error.message);
                if (e) {
                    console.log("Failed to run 'config' mode. Error:", e);
                    return callback(e);
                }
            }
            return callback(null);
        });
    };
    SlGulp.prototype.runScanMode = function (scanModeData, callback) {
        if (!this.isEnabled()) {
            return callback(null);
        }
        if (!scanModeData) {
            scanModeData = new ScanModeData_1.ScanModeData();
        }
        if (!callback) {
            throw new Error("'callback' cannot be null or undefined");
        }
        scanModeData = objectAssign(new ScanModeData_1.ScanModeData(), scanModeData);
        scanModeData.appName = this.appNameResolver.resolve(scanModeData.appName);
        scanModeData.buildSessionIdFile = this.buildSessionIdFileResolver.resolve(scanModeData.buildSessionIdFile);
        scanModeData.buildSessionId = this.buildSessionIdResolver.resolve(scanModeData.buildSessionId, scanModeData.buildSessionIdFile);
        scanModeData.token = this.tokenResolver.resolve(scanModeData.token, scanModeData.tokenFile);
        scanModeData.proxy = this.proxyResolver.resolve(scanModeData.proxy);
        this.slNodeJsProxy.runScanMode(scanModeData, function (error) {
            if (error) {
                var e = gutil.PluginError("gulp-slnodejs", "SlNodeJS failed while running 'scan' mode. Error: " + error.message);
                if (e) {
                    console.log("Failed to run 'scan' mode. Error:", e);
                    return callback(e);
                }
            }
            return callback(null);
        });
    };
    SlGulp.prototype.isEnabled = function () {
        var isEnabled = false;
        if (process.env && process.env["SL_ENABLED"]) {
            isEnabled = process.env["SL_ENABLED"].toLowerCase() === "true";
        }
        return true;
        //return isEnabled;
    };
    return SlGulp;
}());
exports.SlGulp = SlGulp;
//# sourceMappingURL=SlGulp.js.map