"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var bunyan = require("bunyan");
var bformat = require("bunyan-format");
var events_1 = require("events");
var formatStdOut = bformat({ levelInString: true }, process.stdout);
var TRACE = 10;
var DEBUG = 20;
var INFO = 30;
var WARN = 40;
var ERROR = 50;
var FATAL = 60;
var EMPTY_LOGGERS_LIST = [""];
function mapLevelToName(level) {
    switch (level) {
        case TRACE:
            return 'TRACE';
        case DEBUG:
            return 'DEBUG';
        case INFO:
            return 'INFO';
        case WARN:
            return 'WARN';
        case ERROR:
            return 'ERROR';
        case FATAL:
            return 'FATAL';
    }
}
var LoggerFactory = /** @class */ (function () {
    function LoggerFactory(cfg) {
        if (cfg) {
            this.cfg = cfg;
        }
    }
    LoggerFactory.prototype.getCreateBunyanBufferingLogger = function () {
        if (this.bunyanBufferingLogger == null) {
            this.bunyanBufferingLogger = new BunyanBufferingLogger({});
        }
        return this.bunyanBufferingLogger;
    };
    LoggerFactory.prototype.getCreateApplicationLogger = function () {
        var loggerConfig = {
            name: "NodeJS-Agent[" + process.pid + "]",
            streams: []
        };
        var loggers = EMPTY_LOGGERS_LIST;
        if (this.cfg && this.cfg.loggers && this.cfg.loggers.value) {
            loggers = this.cfg.loggers.value.split(',');
        }
        if (process.env.NODE_DEBUG == "sl" || process.env.NODE_DEBUG == "sl-console" || loggers.indexOf('console') >= 0) {
            loggerConfig.streams.push({ level: this.getLogLevel(), stream: formatStdOut });
        }
        if (process.env.NODE_DEBUG == "sl-file" || loggers.indexOf('file') >= 0) {
            loggerConfig.streams.push({ level: this.getLogLevel(), path: "sl.node.log" });
        }
        var remoteStream = this.getCreateBunyanBufferingLogger();
        loggerConfig.streams.push({ level: this.getLogLevel(), type: "raw", stream: remoteStream });
        var bunyanLogger = bunyan.createLogger(loggerConfig);
        return bunyanLogger;
    };
    LoggerFactory.prototype.getLogLevel = function () {
        return process.env.SL_LOG_LEVEL || "info";
    };
    return LoggerFactory;
}());
exports.LoggerFactory = LoggerFactory;
var BunyanBufferingLogger = /** @class */ (function (_super) {
    __extends(BunyanBufferingLogger, _super);
    function BunyanBufferingLogger(config) {
        var _this = _super.call(this) || this;
        _this.config = config;
        _this.acceptLogs = true;
        _this.buffer = [];
        _this.bufferLength = config.bufferLength || 50;
        return _this;
    }
    BunyanBufferingLogger.prototype.write = function (data) {
        if (!this.acceptLogs) {
            return;
        }
        data.level = mapLevelToName(data.level);
        this.buffer.push(data);
        this.checkBuffer(false);
    };
    BunyanBufferingLogger.prototype.processBuffer = function () {
        if (this.buffer.length == 0)
            return false;
        var entries = this.buffer.splice(0, this.buffer.length);
        if (entries.length > 0) {
            this.emit("log_buffer_full", entries);
        }
        return true;
    };
    ;
    BunyanBufferingLogger.prototype.stop = function () {
        this.acceptLogs = false;
    };
    BunyanBufferingLogger.prototype.checkBuffer = function (inShutdown) {
        if (!this.buffer.length) {
            return false; //Nothing more to send
        }
        if (inShutdown || this.buffer.length >= this.bufferLength) {
            return this.processBuffer();
        }
    };
    ;
    return BunyanBufferingLogger;
}(events_1.EventEmitter));
exports.BunyanBufferingLogger = BunyanBufferingLogger;
//# sourceMappingURL=logger.js.map