"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var sl_js_infra_1 = require("sl-js-infra");
var logger_1 = require("./logger");
var getSystemDateValueOf = sl_js_infra_1.SlJsInfra.getSystemDateValueOf;
var fs = require("fs");
//TODO: use all constants and interfaces from infra
var DEFAULT_TEST_NAME = "__init";
var DEFAULT_EXECUTION_ID = "00000000-0000-0000-0000-000000000000";
var NycReportExecutor = /** @class */ (function () {
    function NycReportExecutor(reportFile, agentConfig) {
        if (!reportFile) {
            throw new Error("reportFile must be given");
        }
        if (!agentConfig) {
            throw new Error("agentConfig must be given");
        }
        this.reportFile = reportFile;
        this.agentConfig = agentConfig;
        this.environmentDataService = new sl_js_infra_1.SlJsInfra.EnvironmentDataService(agentConfig);
        this.agentDataService = new sl_js_infra_1.SlJsInfra.AgentDataService();
        this.createLogger();
    }
    NycReportExecutor.prototype.execute = function () {
        try {
            var reportData = this.readReportFile();
            var extendedIstanbulData = this.createExtendedIstanbulData(reportData);
            var footprintsFile = this.convertToFootprints(extendedIstanbulData);
            this.submitFootprints(footprintsFile);
            console.log("Command 'uploadNycReport' finished successfully");
            return true;
        }
        catch (err) {
            console.error("Failed to upload nyc report. error: " + err);
            return false;
        }
    };
    NycReportExecutor.prototype.readReportFile = function () {
        var content = fs.readFileSync(this.reportFile, { encoding: "utf8" });
        if (!content) {
            throw new Error("Cannot read report data from: " + this.reportFile);
        }
        return JSON.parse(content);
    };
    NycReportExecutor.prototype.convertToFootprints = function (extendedIstanbulData) {
        var istanbulToFP = new sl_js_infra_1.SlJsInfra.IstanbulToFootprints(this.agentConfig, this.logger);
        var footprintsFile = this.createFootprintsFile();
        return istanbulToFP.createFootprintsPackageFromRawData(extendedIstanbulData, footprintsFile);
    };
    NycReportExecutor.prototype.submitFootprints = function (footprintsFile) {
        var _this = this;
        var callback = function (err) {
            if (err) {
                _this.logger.error("Failed to upload nyc report. error: ", err);
            }
            else {
                _this.logger.info("nyc report uploaded successfully ", err);
            }
        };
        var footprintsProxy = new sl_js_infra_1.SlJsInfra.FootprintsProxy(this.agentConfig, this.logger);
        footprintsProxy.handleFootprints(footprintsFile, callback);
    };
    NycReportExecutor.prototype.createExtendedIstanbulData = function (rawIstanbulData) {
        return {
            footprints: rawIstanbulData,
            testName: DEFAULT_TEST_NAME,
            executionId: DEFAULT_EXECUTION_ID,
            localTime: getSystemDateValueOf()
        };
    };
    NycReportExecutor.prototype.createFootprintsFile = function () {
        return {
            customerId: this.agentConfig.customerId.value,
            environment: this.environmentDataService.getEnvDataForSubmission(),
            configurationData: this.agentConfig.toJsonObject(),
            meta: {
                sequence: 0,
                generated: getSystemDateValueOf(),
                agentId: this.agentDataService.getAgentId(),
            },
            tests: [],
            apps: []
        };
    };
    NycReportExecutor.prototype.createLogger = function () {
        var factory = new logger_1.LoggerFactory(this.agentConfig);
        this.logger = factory.getCreateApplicationLogger();
    };
    return NycReportExecutor;
}());
exports.NycReportExecutor = NycReportExecutor;
//# sourceMappingURL=nyc-report-executor.js.map