"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var events = require("events");
var timers_1 = require("timers");
var sl_js_infra_1 = require("sl-js-infra");
var INITIAL_COLOR = "00000000-0000-0000-0000-000000000000/__init";
var StateTrackerProcess = /** @class */ (function (_super) {
    __extends(StateTrackerProcess, _super);
    function StateTrackerProcess(cfg, configProcess, stateTrackerProxy, checkTestStatusWatchdog, logger) {
        var _this = _super.call(this) || this;
        _this.cfg = cfg;
        _this.configProcess = configProcess;
        _this.stateTrackerProxy = stateTrackerProxy;
        _this.checkTestStatusWatchdog = checkTestStatusWatchdog;
        _this.logger = logger;
        _this.currentTestIdentifier = null;
        _this._hasAnonymousMappingAtServer = false;
        _this.isRunning = false;
        if (!cfg) {
            throw new Error("cfg is required");
        }
        if (!configProcess) {
            throw new Error("configProcess is required");
        }
        if (!stateTrackerProxy) {
            throw new Error("stateTrackerProxy is required");
        }
        if (!checkTestStatusWatchdog) {
            throw new Error("checkTestStatusWatchdog is required");
        }
        if (!logger) {
            throw new Error("logger is required");
        }
        if (_this.cfg.useInitialColor.value && _this.currentTestIdentifier == null) {
            _this.switchToAnonFootprints();
        }
        configProcess.on("configuration_changed", function (newCfg) {
            if (newCfg.useInitialColor.value && _this.currentTestIdentifier == null) {
                _this.switchToAnonFootprints();
            }
            _this.cfg = newCfg;
        });
        _this.startCheckingTestStatusAtServer();
        return _this;
    }
    StateTrackerProcess.prototype.startCheckingTestStatusAtServer = function () {
        var _this = this;
        if (sl_js_infra_1.SlJsInfra.SlEnvVars.inProductionListenerMode()) {
            this.logger.debug("In production listener, no need to check test status in server");
            return;
        }
        this.checkTestStatusWatchdog.on("alarm", function () {
            _this.checkTestStatusAtServer();
        });
        //Invoke it directly after the CTOR ends.
        timers_1.setTimeout(function () { _this.checkTestStatusAtServer(); }, 0);
    };
    StateTrackerProcess.prototype.checkTestStatusAtServer = function () {
        var _this = this;
        if (this.currentTestIdentifier == INITIAL_COLOR) {
            //check if there is still a mapping
            this.stateTrackerProxy.checkMappingStatus(function (err, hasMapping) {
                if (err) {
                    _this.logger.warn("Error while checking test execution status " + err);
                }
                else { //Agent was in "initial color" and now the server returns that the mapping is no longer active
                    _this._hasAnonymousMappingAtServer = hasMapping;
                }
            });
        }
        else {
            this.logger.info("'currentTestIdentifier' is null. That means that footprints will not be sent.");
        }
    };
    StateTrackerProcess.prototype.hasAnonymousMappingAtServer = function () {
        return this._hasAnonymousMappingAtServer;
    };
    StateTrackerProcess.prototype.isAnonymousColor = function (testIdentifier) {
        return INITIAL_COLOR == testIdentifier;
    };
    StateTrackerProcess.prototype.switchToAnonFootprints = function () {
        this.logger.info("Switching to anonymous footprints.");
        this.setTestIdentifier(INITIAL_COLOR, false);
    };
    StateTrackerProcess.prototype.getCurrentTestIdentifier = function () {
        return this.currentTestIdentifier;
    };
    StateTrackerProcess.prototype.setTestIdentifier = function (newTestIdentifier, silent) {
        this.logger.info("setting test identifier: " + newTestIdentifier);
        if ((this.currentTestIdentifier == null) && (newTestIdentifier != null) || (this.currentTestIdentifier != null && this.currentTestIdentifier != newTestIdentifier && newTestIdentifier != INITIAL_COLOR)) {
            var previousTestIdentifier = this.currentTestIdentifier;
            this.currentTestIdentifier = newTestIdentifier;
            if (this.isRunning) {
                if (!silent) {
                    this.emit("test_identifier_changed", this.currentTestIdentifier, previousTestIdentifier);
                }
                else {
                    this.logger.info("Test identifier changed, running in silent mode not enqueuing footprints");
                }
                this.checkTestStatusWatchdog.reset();
            }
        }
        else {
            this.logger.info("Not setting the color. newTestIdentifier is '" + newTestIdentifier + "' " + "and currentTestIdentifier is '" + this.currentTestIdentifier + "'");
        }
    };
    StateTrackerProcess.prototype.start = function () {
        if (this.isRunning) {
            return;
        }
        this.checkTestStatusWatchdog.start();
        this.isRunning = true;
    };
    StateTrackerProcess.prototype.stop = function (callback) {
        try {
            if (!this.isRunning) {
                return callback();
            }
            this.checkTestStatusWatchdog.stop();
            this.isRunning = false;
            callback();
        }
        catch (err) {
            //TODO: log
            return callback();
        }
    };
    StateTrackerProcess.splitTestIdToExecutionAndTestName = function (testId) {
        testId = testId || '';
        var executionId = testId.split('/')[0];
        var testName = (testId.indexOf('/') == -1) ? "" : testId.substring(executionId.length + 1);
        if (!executionId) {
            testName = "";
        }
        return { executionId: executionId, testName: testName };
    };
    StateTrackerProcess.combineExecutionIdAndTestName = function (executionId, testName) {
        testName = testName || '';
        if (!executionId) {
            return '';
        }
        return executionId + '/' + testName;
    };
    return StateTrackerProcess;
}(events.EventEmitter));
exports.StateTrackerProcess = StateTrackerProcess;
//# sourceMappingURL=index.js.map