"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var path = require("path");
var fs = require("fs");
var async = require('async');
var logger_1 = require("./logger");
var UploadReportHandler = /** @class */ (function () {
    function UploadReportHandler(args, backendProxy, agentData) {
        this.reportRequest = args;
        this.backendProxy = backendProxy;
        this.agentData = agentData;
        this.logger = new logger_1.LoggerFactory().getCreateApplicationLogger();
    }
    UploadReportHandler.prototype.upload = function (callback) {
        var _this = this;
        try {
            var files = this.getAllFiles(this.reportRequest.reportFile, this.reportRequest.reportFilesFolder);
            if (files.length == 0) {
                throw "Couldn't find any reports files in '" + this.reportRequest.reportFilesFolder + "'.\n                       Please specify a folder with valid report files. 'UploadReport' action was aborted";
            }
            var context_1 = this;
            async.forEachLimit(files.slice(0, -1), 1, function (reportFile, next) {
                var agentData = _this.agentData;
                agentData.hasMoreRequests = true;
                var requestBody = _this.createRequestBody(agentData, reportFile);
                _this.backendProxy.uploadReport(requestBody, function (err) {
                    if (err) {
                        next(err);
                    }
                    else {
                        next(null);
                    }
                });
            }, function (err) {
                if (err) {
                    return callback(err);
                }
                var lastFile = files[files.length - 1];
                var requestBody = context_1.createRequestBody(context_1.agentData, lastFile);
                context_1.backendProxy.uploadReport(requestBody, function (err) {
                    if (err) {
                        callback(err);
                    }
                    else {
                        callback(null);
                    }
                });
            });
        }
        catch (err) {
            callback(err);
        }
    };
    UploadReportHandler.prototype.createRequestBody = function (agentData, reportFile) {
        var requestBody = {
            agentData: agentData,
            reportFile: reportFile
        };
        return requestBody;
    };
    UploadReportHandler.prototype.getAllFiles = function (reportFiles, reportFilesFolders) {
        var _this = this;
        var files = [];
        // handle reportFilesFolders
        reportFilesFolders.forEach(function (reportFilesFolder) {
            try {
                var folderFiles = fs.readdirSync(reportFilesFolder);
                folderFiles.forEach(function (folderFileName) {
                    var stat = fs.statSync(path.join(reportFilesFolder, folderFileName));
                    if (stat.isFile()) {
                        files.push({ filename: folderFileName, buffer: fs.readFileSync(path.join(reportFilesFolder, folderFileName)) });
                    }
                    else {
                        this.logger.debug("[UPLOAD REPORTS] " + folderFileName + " is a directory. Ignoring...");
                    }
                });
            }
            catch (err) {
                _this.logger.error("[UPLOAD REPORTS] Failed Reading Folder: " + reportFilesFolder);
            }
        });
        // handle reportFiles
        reportFiles.forEach(function (reportFilePath) {
            if (fs.existsSync(reportFilePath)) {
                files.push({ filename: path.basename(reportFilePath), buffer: fs.readFileSync(reportFilePath) });
            }
            else {
                _this.logger.error("[UPLOAD REPORTS] reportFile doesn't exist. report file: " + reportFilePath);
            }
        });
        return files;
    };
    ;
    return UploadReportHandler;
}());
exports.UploadReportHandler = UploadReportHandler;
//# sourceMappingURL=upload-report-handler.js.map