"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var WaitForMultipleCallbacks = /** @class */ (function () {
    function WaitForMultipleCallbacks() {
        this.processStatuses = {};
    }
    WaitForMultipleCallbacks.prototype.waitForAll = function (callback) {
        if (this.checkCompletion())
            return callback(); //Already done (happens when all callbacks have been synchronously called)
        else {
            if (this.completionCallback) {
                throw new Error("callback already set");
            }
            this.completionCallback = callback; //Not done yet. This callback will be invoked later in checkCompletionAndFireCallback()
        }
    };
    /**
     * Creates a named callback and waits for it to finish (asynchronously)
     * @param {string} name
     * @returns
     *
     * @memberOf WaitForMultipleCallbacks
     */
    WaitForMultipleCallbacks.prototype.getCallbackForProcess = function (name) {
        var _this = this;
        if (this.processStatuses.hasOwnProperty(name))
            throw new Error("Duplicate process name: " + name);
        this.processStatuses[name] = false;
        return function () {
            _this.processStatuses[name] = true;
            _this.checkCompletionAndFireCallback();
        };
    };
    WaitForMultipleCallbacks.prototype.checkCompletionAndFireCallback = function () {
        if (this.checkCompletion()) {
            if (this.completionCallback) {
                var cb = this.completionCallback;
                this.completionCallback = null;
                cb();
            }
        }
    };
    /**
     * Checks if all callbacks have been called
     *
     * @returns true if all processes have been completed, false if some callbacks have yet to be called
     *
     * @memberOf WaitForMultipleCallbacks
     */
    WaitForMultipleCallbacks.prototype.checkCompletion = function () {
        for (var name in this.processStatuses) {
            if (this.processStatuses[name] === false)
                return false;
        }
        return true; //All done
    };
    return WaitForMultipleCallbacks;
}());
exports.WaitForMultipleCallbacks = WaitForMultipleCallbacks;
//# sourceMappingURL=wait-for-multiple-callbacks.js.map